import os
import requests
import simplejson as json
from pmb_py import PmbError

_session = None

AUTH_URL = os.environ['AUTH_URL']
PMBAPI_URL = os.environ['PMB_API_URL']


class Session:
    service_root = PMBAPI_URL
    if os.environ.get('DEBUG') in ('True', 'true', True):
        service_root = os.environ.get('PMBAPI_URL_DEBUG', 'http://127.0.0.1:5000') 
    headers = {'user-agent': 'Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:47.0) Gecko/20100101 Firefox/47.0',
               'content-type': 'application/json'}

    def __init__(self):
        self.cookies = requests.cookies.RequestsCookieJar()
        self.user = None
        self.auth_status = False

    def authorization(self, user, pw):
        '''get token, presver token in cookie

        Args:
            user(str): user name
            pw(str): password

        Returns:
            bool: bool
        '''
        url = '/'.join([AUTH_URL, 'auth', 'api', 'login'])
        try:
            resp = requests.post(url, auth=(user, pw), headers=self.headers)
        except requests.ConnectionError as _:
            raise PmbError(
                'Services seems not available, Please connect administrator')

        if resp.status_code == 200 and resp.text:
            cookies_data = resp.headers.get('Set-Cookie')
            if cookies_data:
                self.__set_cookie(resp.headers.get('Set-Cookie'))
            else:
                cookies = {'auth_token=': resp.text}
                self.cookies = cookies
            self.headers['Cookie'] = 'auth_token=' + resp.text
            self.auth_status = True
            self.user = user
            return True
        else:
            return False

    def __set_cookie(self, set_cookie_info):
        if not set_cookie_info:
            return

        cookie_item = set_cookie_info.split(';')[0]
        cookie_value = cookie_item.split('=')[1]
        self.cookies.set('ci_session', cookie_value, path='/')

    def get(self, api, params={}, kwargs={}):
        return self.__request('get', api, params, kwargs=kwargs)

    def post(self, api, params={}, data={}, kwargs={}):
        return self.__request('post', api, params, data, kwargs)

    def put(self, api, params={}, data={}, kwargs={}):
        return self.__request('put', api, params, data, kwargs)

    def patch(self, api, params={}, data={}, kwargs={}):
        return self.__request('patch', api, params, data, kwargs)

    def delete(self, api, params={}, data={}, kwargs={}):
        return self.__request('delete', api, params, data, kwargs)

    def __request(self, method, api, params={}, data={}, kwargs={}):
        url = '/'.join([self.service_root, api])
        dumped_data = json.dumps(data)
        request_kargs = {
            'headers': self.headers,
            'cookies': self.cookies,
            'params': params,
            'data': dumped_data
        }
        if kwargs:
            request_kargs['allow_redirects'] = kwargs.get(
                'allow_redirects', True)

        resp = requests.request(method, url, **request_kargs)

        self.__check_response_stat(resp)
        # print(resp.text)
        j_obj = self.__parse_json(resp.text)
        return j_obj

    def __check_response_stat(self, resp):
        if resp.status_code not in [requests.codes['ok'], requests.codes['created']]:
            if resp.status_code == 500:
                msg = "The Monkey working in server do some thing wrong:"
                msg += resp.text
                raise PmbError(msg)
            else:
                msg = 'status: {}, {}'.format(resp.status_code, resp.text)
                raise PmbError(msg)

    def __parse_json(self, json_str):
        try:
            j_obj = json.loads(json_str)
            return j_obj
        except ValueError as e:
            raise PmbError(e)

    def log_out(self):
        url = '/'.join([AUTH_URL, 'logout'])
        requests.post(url)
        global _session
        _session = None
        return True


def log_in(ac, pw):
    from pmb_py.core import Session
    global _session
    if _session and _session.user:
        return _session
    else:
        _session = Session()
        _session.authorization(ac, pw)
        return True


def log_out():
    global _session
    if _session and _session.user:
        return _session.log_out()
    else:
        raise PmbError('Not In Log-in Status')



import pmb_py


def test_list_gantt_item(pmb):
    objs = pmb.api.GanttItems.list()
    assert objs


def test_create_gantt_item(pmb):
    obj = pmb.api.GanttItems.create(name='test1')
    assert type(obj) == pmb_py.domain.model.PmbGanttItem


def test_list_gantt_item_with_kwarg_name(pmb):
    objs = pmb.api.GanttItems.list(name='test1')
    for obj in objs:
        assert obj.name == 'test1'


def test_update_gantt_item(pmb):
    obj = pmb.api.GanttItems.get(name='test1')
    assert obj.name == 'test1'
    obj = pmb.api.GanttItems.update(obj.id, sg_task_id=100)
    assert obj.sg_task_id == 100



PMB_MEMBER_STATUS = {
    "leave": [0, '已離職'],
    "atWork": [1, '在職中'],
    "leaveOfAbsence": [2, '留職停薪'],
}


class PmbError(Exception):
    pass


from pmb_py.core import log_in, log_out
import pmb_py.api as api


if __name__ == '__main__':
    import os
    print(log_in(os.environ['TESTUSER'], os.environ['TESTPW']))
    print(api.ProjectStatus.list())
    print(api.MemberStatus.list())
    print(api.ProjectType.list())
    print(api.Projects.list())
    print(api.GanttItems.list())


import pmb_py.core as core
from pmb_py import PMB_MEMBER_STATUS, PmbError
from pmb_py.adapter import pmb_convert


class pmb_mixin:
    creator = None

    @classmethod
    def list(cls, *args, **kwargs):
        items = cls.get_query(*args, **kwargs)
        if not items: return dict()
        if cls.creator:
            objects = [cls.creator(item) for item in items]
            return {object.id: object for object in objects}.values()
        else:
            return items

    @classmethod
    def get_query(cls, *args, **kwargs):
        pass

    @classmethod
    def get(cls, **kwargs):
        pass
        # raise PmbError("no support for these keyword")

    @classmethod
    def create_one(cls, *args, **kwargs):
        return dict()

    @classmethod
    def create(cls, *args, **kwargs):
        item = cls.create_one(*args, **kwargs)
        if cls.creator:
            obj = cls.creator(item)
            return obj
        else:
            return item

    @classmethod
    def update_one(cls, obj_id, **kwargs):
        return dict()

    @classmethod
    def update(cls, obj_id, **kwargs):
        try:
            item = cls.update_one(obj_id, **kwargs)
        except Exception as e:
            print(str(e))
            return False
        if cls.creator:
            obj = cls.creator(item)
            return obj
        else:
            return item


class ProjectStatus(pmb_mixin):
    creator = pmb_convert.to_project_status

    @classmethod
    def get_query(cls):
        api = "web_project_status_and_types"
        re = core._session.post(api)
        return re["data"]["projectStatus"]


class ProjectType(pmb_mixin):
    creator = pmb_convert.to_project_type

    @classmethod
    def get_query(cls):
        api = "web_project_status_and_types"
        re = core._session.post(api)
        return re["data"]["projectTypes"]


class MemberStatus(pmb_mixin):
    creator = pmb_convert.to_member_status

    @classmethod
    def get_query(cls):
        status_list = list()
        for k, v in PMB_MEMBER_STATUS.items():
            status_list.append({"id": v[0], "name": v[1], "codeName": k})
        return status_list


class Projects(pmb_mixin):
    creator = pmb_convert.to_project

    @classmethod
    def get_query(cls, *args, **kwargs):
        api = 'projects'
        re = core._session.get(api, params=kwargs)
        return re


# class PmbTasks(pmb_mixin):
#     _cache = None
#     creator = pmb_convert.to_pmb_task


class Blocks(pmb_mixin):
    creator = pmb_convert.to_pmb_block

    @classmethod
    def list(cls, **kwargs):
        raise PmbError(
            "PmbBlocks is not support list method, instead, use get(id=) or get (task_id=)"
        )

    @classmethod
    def get_query(cls, *args, **kwargs):
        api = "web_tasks_logs_a_day"

    @classmethod
    def get(cls, **kwargs):
        blocks = list()
        task_id = kwargs.get("task_id")
        if not task_id:
            raise PmbError("task_id is require")
        date_ = kwargs.get("task_id")
        if not date_:
            raise PmbError('date is require')
        data = {
            
        }


class GanttItems(pmb_mixin):
    creator = pmb_convert.to_pmb_gantt_item

    @classmethod
    def get_query(cls, *args, **kwargs):
        api = 'gantt_items'
        re = core._session.get(api, params=kwargs)
        return re

    @classmethod
    def create_one(cls, *args, **kwargs):
        api = 'gantt_items'
        re = core._session.post(api, data=kwargs)
        return re


from pmb_py.domain import model


def to_project_status(item):
    return model.ProjectStatus(item['id'], item['name'], item['colorCode'], item['code'], item['colorText'])


def to_project_type(item):
    return model.ProjectType(item['id'], item['name'])


def to_member_status(item):
    return model.MemberStatus(item['id'], item['name'], item['codeName'])


def to_project(item):
    return model.Project(**item)


# def to_pmb_task(item):
#     return model.


def to_pmb_block(item):
    return model.Block(item['id'], item['date'], item['duration'], item['task_id'], item['member_id'], item['project_id'], item['type_id'])


def to_pmb_gantt_item(item):
    return model.GanttItem(id=item['id'], name=item['name'], start_datetime=item['start_datetime'], end_datetime=item['end_datetime'],
                           task_type=item['task_type'], parent_id=item['parent_id'], sg_task_id=item['sg_task_id'], legend=item['legend'],
                           read_only=item['read_only'], status=item['status'], project_id=item['project_id']
                           )



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pmb_py",
    version="0.0.7",
    author="Noflame.lin",
    author_email="linjuang@gmail.com",
    description="pmb restful api python wrap",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/MoonShineVFX/pmb_py_api",
    install_requires=['requests', 'simplejson'],
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.6",
)


from typing import List, Union
from dataclasses import dataclass
import datetime


@dataclass
class ProjectStatus:
    id: int
    name: str
    color: str
    code_name: str
    color_text: str


class Project:

    def __init__(self, id: int,
                 name: str,
                 alive: bool,
                 budget: float=None,
                 budget_tax: float=None,
                 cost: float=None,
                 profit: float=None,
                 money_received: float=None,
                 deadline: datetime.date=None,
                 status_id: int=None,
                 note: str = '',
                 time_start: datetime.date=None,
                 time_end: datetime.date=None,
                 time_update: datetime.date=None,
                 sort_num: float=None,
                 manager_id: int=None,
                 manager_assistant_id: int=None,
                 leader_id: int=None,
                 leader_agent_id: int=None,
                 director_id: int=None,
                 producer_id: int=None,
                 cg_lead_id: int=None,
                 project_coordinator_id: int=None,
                 extra_pm_id: int=None,
                 charge: float=None,
                 old_company: str=None,
                 payment_status_id: int=None,
                 company_id: int=None,
                 sg_project_id: int=None,
                 contracted: bool=None,
                 is_deleted: bool=None,
                 tank_name: str=None,
                 backup_unc: str=None,
                 type_id: int = None):
        self.id = id
        self.name = name
        self.type_id = type_id
        self.budget = budget
        self.budget_tax = budget_tax
        self.cost = cost
        self.profit = profit
        self.money_received = money_received
        self.alive = alive
        self.deadline = deadline
        self.status_id = status_id
        self.note = note
        self.time_start = time_start
        self.time_end = time_end
        self.time_update = time_update
        self.sort_num = sort_num

        self.manager_id = manager_id
        self.manager_assistant_id = manager_assistant_id
        self.leader_id = leader_id
        self.leader_agent_id = leader_agent_id
        self.director_id = director_id
        self.producer_id = producer_id
        self.cg_lead_id = cg_lead_id
        self.project_coordinator_id = project_coordinator_id
        self.extra_pm_id = extra_pm_id

        self.charge = charge
        self.old_company = old_company
        self.payment_status_id = payment_status_id
        self.company_id = company_id
        self.sg_project_id = sg_project_id
        self.contracted = contracted
        self.is_deleted = is_deleted
        self.tank_name = tank_name
        self.backup_unc = backup_unc

    def __eq__(self, obj_b):
        if type(obj_b) != PmbProject:
            return False
        if self.id == obj_b.id:
            return True
        else:
            return False

    def __hash__(self):
        data = 'PmbProject_'+str(self.id)
        return hash(data)

    def __repr__(self):
        class_ = str(type(self)).split("'")[1]
        return f'{class_}({self.id}:{self.name}) at {id(self)}'


@dataclass
class ProjectType:
    id: int
    name: str


@dataclass
class MemberStatus:
    id: int
    name: str
    code_name: str


@dataclass
class Member:
    id: int
    name: str
    e_id: str
    email: str


@dataclass
class Task:
    id: int
    name: str
    gantt_item_id: int


@dataclass
class Block:
    id: int
    date: datetime.date
    duration: int
    task_id: int
    member_id: int
    project_id: int
    type_id: int


@dataclass
class BlockType:
    id: int
    name: str
    color: str
    color_text: str
    text: str


@dataclass
class GanttItem:
    id: int
    name: str
    start_datetime: datetime.datetime
    end_datetime: datetime.datetime
    task_type: int
    parent_id: int
    project_id:int
    sg_task_id: int
    legend: str
    read_only: str
    status: str



@dataclass
class Member:
    id: int
    emp_id: str
    name: str
    email: str

